/* SNOOPER.c								tab:4
 *
 * Copyright (c) 2001 the University of Southern California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF SOUTHERN CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF SOUTHERN CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF SOUTHERN CALIFORNIA HAS NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 *
 * listens all packets and pass them to UART.
 * Support different packet length. The first byte must be packet length.
 * Data is received from radio and passed to UART on a per byte basis.
 * If on a per packet basis, a short packet following a long packet may get
 * lost because the UART can't finish sending the long packet when the short
 * packet arrives.
 * The contents of each packet can be displayed by snoope.c at tools/.
 *
 * Authors:   Wei Ye (USC/ISI)
 *
 */

#include "tos.h"
#include "phy_radio_msg.h"
#include "SNOOPER.h"

// By defining the following micro, snooper can show the CRC check result
// of each packet by appending an extra byte 'error' at the end of each pkt.
// 0 - no error, 1 - error. The packet length shown will be increased by 1.
//#define SHOW_ERR_CHECK

#define TOS_FRAME_TYPE SNOOPER_frame
TOS_FRAME_BEGIN(SNOOPER_frame) {
#ifdef SHOW_ERR_CHECK
	unsigned char txCount;
#endif
/*
	unsigned char pktLen;
	char txByte;
	char bufByte;
*/
	char bufByte;
	char flagUart;
	char flagBuf;
}
TOS_FRAME_END(SNOOPER_frame);


char TOS_COMMAND(SNOOPER_INIT)(){
	VAR(flagUart) = 0;
	VAR(flagBuf) = 0;
	TOS_CALL_COMMAND(SNOOPER_SUB_UART_INIT)();
    TOS_CALL_COMMAND(SNOOPER_SUB_PHY_INIT)();
    return 1;
} 


char TOS_COMMAND(SNOOPER_START)(){
	return 1;
}

char TOS_EVENT(SNOOPER_PHY_START_SYM_DETECTED)(void) {
//	VAR(txByte) = TOS_UART_ADDR;
#ifdef SHOW_ERR_CHECK
	VAR(txCount) = 0;
#endif
	TOS_CALL_COMMAND(SNOOPER_SUB_UART_TX_BYTES)(TOS_UART_ADDR);
	return 1;
}

char TOS_EVENT(SNOOPER_PHY_RX_BYTE_DONE)(char data)
{
	// suppose UART speed is faster than data arrival rate from radio
/*
	// put byte into buffer
	VAR(bufByte) = data;
	if (VAR(txCount) == 0) {
		VAR(pktLen) = (unsigned char)data + 1;  // count in TOS_UART_ADDR
#ifdef SHOW_ERR_CHECK
		// append error check result at end of pkt
		VAR(bufByte)++;
		VAR(pktLen)++;
#endif
	}
*/
	// send byte in tx buffter
#ifdef SHOW_ERR_CHECK
	if (VAR(txCount) == 0) {
		VAR(txCount) = 1;
		data++;
	}
#endif
	if (VAR(flagUart) == 0) {
		VAR(flagUart) = 1;
		TOS_CALL_COMMAND(SNOOPER_SUB_UART_TX_BYTES)(data);
	} else if (VAR(flagBuf) == 0) {
		VAR(bufByte) = data;
		VAR(flagBuf) = 1;
	}
		
	return 1;
}

char TOS_EVENT(SNOOPER_UART_TX_BYTE_READY)(char success){
/*
	VAR(txCount)++;
	VAR(txByte) = VAR(bufByte);
	// check if reaches last byte
	if (VAR(txCount) == VAR(pktLen) - 1) {
		TOS_CALL_COMMAND(SNOOPER_SUB_UART_TX_BYTES)(VAR(txByte));
	}
*/
	if (VAR(flagBuf) == 1) {
		TOS_CALL_COMMAND(SNOOPER_SUB_UART_TX_BYTES)(VAR(bufByte));
		VAR(flagBuf) = 0;
	} else {
		VAR(flagUart) = 0;
	}
	
	return 1; 
}

char TOS_EVENT(SNOOPER_UART_RX_BYTE_READY)(char data, char error) {
	return 1;
}

char TOS_EVENT(SNOOPER_PHY_TX_PKT_DONE)(void* packet) {
	return 1;
}

void* TOS_EVENT(SNOOPER_PHY_RX_PKT_DONE)(void* packet, char error) {
	if (packet == 0) {
		TOS_CALL_COMMAND(SNOOPER_SUB_UART_TX_BYTES)(0); // stop snoop.c
#ifdef SHOW_ERR_CHECK
	} else {
		TOS_CALL_COMMAND(SNOOPER_SUB_UART_TX_BYTES)(error);
#endif
	}
	return packet;
}

char TOS_EVENT(SNOOPER_PHY_CHANNEL_IDLE)(void) {
	return 1;
}

char TOS_EVENT(SNOOPER_PHY_CHANNEL_BUSY)(void) {
	return 1;
}
