/* CODEC_MANCHESTER.c                                            tab:4
 *
 * Copyright (c) 2002 the University of Southern California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF SOUTHERN CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF SOUTHERN CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF SOUTHERN CALIFORNIA HAS NO
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 *
 *
 * Authors:  Wei Ye (USC/ISI)
 * Date created: July 21, 2002
 *
 * This module implements Manchester encoding and decoding
 *
 */


#include "tos.h"
#include "CODEC_MANCHESTER.h"
#include "dbg.h"

#define IDLE_STATE 0
#define DECODING_BYTE 1 

//table for performing the encoding.
static const char encodeTab[16] =
{0x55,
 0x56,
 0x59,
 0x5a,
 0x65,
 0x66,
 0x69,
 0x6a,
 0x95,
 0x96,
 0x99,
 0x9a,
 0xa5,
 0xa6,
 0xa9,
 0xaa};

/* Frame of the component */
#define TOS_FRAME_TYPE codec_frame
TOS_FRAME_BEGIN(codec_frame) {
  char state;
  char error;
  unsigned char decodedVal;
  unsigned char bitVal;
}
TOS_FRAME_END(codec_frame);

char radio_decode_thread(char data);

char TOS_COMMAND(CODEC_INIT)(){
	VAR(state) = IDLE_STATE;
	return 1;
}


char TOS_COMMAND(CODEC_DECODE)(char data){
	if(VAR(state) == IDLE_STATE){
		VAR(state) = DECODING_BYTE;
		VAR(decodedVal) = 0;
		VAR(bitVal) = 0x80;
		VAR(error) = radio_decode_thread(data);
		return 1;
	}else if(VAR(state) == DECODING_BYTE){
		VAR(error) |= radio_decode_thread(data);
     	TOS_SIGNAL_EVENT(CODEC_DECODE_DONE)(VAR(decodedVal), VAR(error));
		VAR(state) = IDLE_STATE;
		return 1;
	}
	return 0;	
}


char TOS_COMMAND(CODEC_ENCODE_FLUSH)(){
	return 1;
}

char TOS_COMMAND(CODEC_ENCODE)(char d){
	char high, low;
	
	if(VAR(state) == IDLE_STATE){
		low = encodeTab[(int)(d & 0xf)];
		high = encodeTab[(int)((d >> 4) & 0xf)];
		TOS_SIGNAL_EVENT(CODEC_ENCODE_DONE)(high); // high byte is sent first
		TOS_SIGNAL_EVENT(CODEC_ENCODE_DONE)(low);
		return 1;
	}
	return 0;	
}

char radio_decode_thread(char data){
	unsigned char mask, i, temp;
	char error = 0;
	mask = 0x80;
	for (i = 0; i < 4; i++) {
		temp = data & mask;
		if((data << 1 & mask) == temp) error = 1;
		if (temp) VAR(decodedVal) += VAR(bitVal);
		VAR(bitVal) >>= 1;
		mask >>= 2;
	}
	return error;
}
